#
# context must be a demand object.
#

from Products.ERP5Type.DateUtils import addToDate

if not context.getPortalType() in ('Purchase Demand', 'Sale Demand', 'Internal Demand'):
  return

resource_value = context.getResourceValue()
time_second_ratio = resource_value.getQuantityUnitDefinitionRatio(context.portal_categories.quantity_unit.time.second)

def createMovementListIter(resource_value,
                           conversion_ratio,
                           order_quantity_unit_value,
                           required_amount,
                           date,
                           dict_list,
                           order_number,
                           min_delay_second,
                           max_delay_second,
                           min_flow,
                           min_flow_second,
                           max_flow,
                           max_flow_second,
                           remaining_flow,
                           min_order,
                           min_order_lack_amount,
                           max_order,
                           remaining_orderable):
  # All quantity is in the default quantity unit of the target resource.
  order_amount = required_amount
  next_date = date

  order_amount = min(max(order_amount, min_order_lack_amount), remaining_flow, remaining_orderable)

  min_order_lack_amount -= order_amount
  remaining_orderable -= order_amount
  remaining_flow -= order_amount

  delay_second = max_delay_second or min_delay_second or 0
  start_date = date
  stop_date = addToDate(start_date, second=delay_second)

  dict_list.append(dict(
    resource_value=resource_value,
    order_number=order_number,
    start_date=start_date,
    stop_date=stop_date,
    quantity=order_amount * conversion_ratio,
    quantity_unit_value=order_quantity_unit_value
  ))

  remaining_amount = required_amount - order_amount

  if remaining_amount == required_amount:
    raise ValueError, "Conditions are inconsistent"

  remaining_amount = max(min_order_lack_amount, remaining_amount)

  if remaining_orderable == 0:
    order_number += 1
    remaining_orderable = max_order
    min_order_lack_amount = min_order
  elif remaining_orderable < 0:
    raise ValueError, remaining_orderable

  if remaining_flow == 0:
    next_date = addToDate(next_date, second=max_flow_second)
    remaining_flow = max_flow
  elif remaining_flow < 0:
    raise ValueError, remaining_flow

  return remaining_amount, remaining_flow, next_date, dict_list, order_number, min_order_lack_amount, remaining_orderable


def createMovementList(resource_value,
                       default_quantity_unit_value,
                       order_quantity_unit_value,
                       time_quantity_unit_value,
                       required_amount,
                       date,
                       min_delay,
                       max_delay,
                       min_flow,
                       max_flow,
                       min_order,
                       max_order):

  min_delay_second = 0
  max_delay_second = 0

  # min_flow is not supported yet.
  min_flow_in_default_quantity_unit = None
  min_flow_second = 0

  if max_flow:
    max_flow_in_default_quantity_unit = max_flow
  else:
    max_flow_in_default_quantity_unit = float('inf')
  max_flow_second = 0
  min_order_in_default_quantity_unit = 0
  max_order_in_default_quantity_unit = float('inf')

  default_quantity_unit_relative_url = default_quantity_unit_value.getCategoryRelativeUrl()

  if time_quantity_unit_value is not None:
    time_second_conversion_ratio = resource_value.getQuantityUnitDefinitionRatio(time_quantity_unit_value) / time_second_ratio

    if min_delay:
      min_delay_second = min_delay * time_second_conversion_ratio
    if max_delay:
      max_delay_second = max_delay * time_second_conversion_ratio

    min_flow_second = time_second_conversion_ratio
    max_flow_second = time_second_conversion_ratio

  if order_quantity_unit_value is not None:
    order_quantity_unit_relative_url = order_quantity_unit_value.getCategoryRelativeUrl()
    order_quantity_unit_default_quantity_unit_conversion_ratio = resource_value.convertQuantity(1, order_quantity_unit_relative_url, default_quantity_unit_relative_url)
    if min_order:
      min_order_in_default_quantity_unit = min_order * order_quantity_unit_default_quantity_unit_conversion_ratio
    if max_order:
      max_order_in_default_quantity_unit = max_order * order_quantity_unit_default_quantity_unit_conversion_ratio
    default_quantity_unit_order_quantity_unit_conversion_ratio = 1 / order_quantity_unit_default_quantity_unit_conversion_ratio
  else:
    default_quantity_unit_order_quantity_unit_conversion_ratio = 1


  dict_list = []
  order_number = 0
  remaining_flow = max_flow_in_default_quantity_unit
  min_order_lack_amount = min_order_in_default_quantity_unit
  remaining_orderable = max_order_in_default_quantity_unit
  while True:
    (required_amount, remaining_flow, date, dict_list, order_number,
     min_order_lack_amount, remaining_orderable
    ) = createMovementListIter(
      resource_value,
      default_quantity_unit_order_quantity_unit_conversion_ratio,
      order_quantity_unit_value,
      required_amount,
      date,
      dict_list,
      order_number,
      min_delay_second,
      max_delay_second,
      min_flow_in_default_quantity_unit,
      min_flow_second,
      max_flow_in_default_quantity_unit,
      max_flow_second,
      remaining_flow,
      min_order_in_default_quantity_unit,
      min_order_lack_amount,
      max_order_in_default_quantity_unit,
      remaining_orderable
    )

    if required_amount <= 0:
      return dict_list

return createMovementList(
  resource_value,
  resource_value.getDefaultQuantityUnitValue(),
  context.getOrderQuantityUnitValue(),
  context.getTimeQuantityUnitValue(),
  context.getQuantity(),
  context.getStartDate(),
  context.getMinDelay(),
  context.getMaxDelay(),
  context.getMinFlow(),
  context.getMaxFlow(),
  context.getMinOrderQuantity(),
  context.getMaxOrderQuantity())
