from Products.ERP5Type.tests.ERP5TypeTestCase import ERP5TypeTestCase
import transaction
from DateTime import DateTime


class AutomatedOrderingPurchaseTest(ERP5TypeTestCase):
  """
  """

  def getTitle(self):
    return "AutomatedOrderingPurchaseTest"

  def getBusinessTemplateList(self):
    """
    Tuple of Business Templates we need to install
    """
    return ('erp5_base',)

  def commitTic(self):
    transaction.commit()
    self.tic()

  def setUpInitialStock(self):
    internal_packing_list_module = self.portal.internal_packing_list_module
    delete_id_list = []
    for brain in self.portal.portal_catalog(
        portal_type='Internal Packing List',
        id='automated_ordering_test%'):
      delete_id_list.append(brain.getId())

    self.portal.internal_packing_list_module.manage_delObjects(ids=delete_id_list)
    self.commitTic()

    self.assertEqual(
      self.portal.portal_simulation.getFutureInventory(
        at_date=DateTime('3000/08/01 00:00:00 GMT+9'),
        resource_uid=self.resource.getUid(),
        section_uid=self.company.getUid(),
        node_uid=self.warehouse.getUid()),
      0)
    self.assertEqual(
      len(self.portal.portal_simulation.getFutureInventoryList(
        resource_uid=self.resource.getUid(),
        section_uid=self.company.getUid(),
        node_uid=self.warehouse.getUid(),
        group_by_movement=1)),
      0)

    initial_stock_data = (
      (DateTime('3000/08/01 00:00:00 GMT+9'),   500, 'delivered'),
      (DateTime('3000/10/02 00:00:00 GMT+9'),   340, 'confirmed'),
      (DateTime('3000/10/05 00:00:00 GMT+9'), -1800, 'confirmed'),
      (DateTime('3000/10/06 00:00:00 GMT+9'),  2000, 'confirmed'),
      (DateTime('3000/10/07 00:00:00 GMT+9'), -2000, 'confirmed'),
      (DateTime('3000/10/10 00:00:00 GMT+9'),  -600, 'confirmed'),
      (DateTime('3000/11/28 00:00:00 GMT+9'), -1300, 'confirmed'),
    )

    for index, (date, quantity, state) in enumerate(initial_stock_data):
      if quantity > 0:
        kw = dict(destination_section_value=self.company,
                  destination_value=self.warehouse)
      else:
        kw = dict(source_section_value=self.company,
                  source_value=self.warehouse)
        quantity = -quantity
      
      packing_list = internal_packing_list_module.newContent(
        id='automated_ordering_test_{}'.format(index),
        start_date=date,
        **kw)
      packing_list.newContent(
        portal_type='Internal Packing List Line',
        resource_value=self.resource,
        quantity_unit_value=self.unit.piece,
        quantity=quantity
      )
      if state == 'delivered':
        packing_list.confirm()
        packing_list.stop()
        packing_list.deliver()
      elif state == 'confirmed':
        packing_list.confirm()

    self.commitTic()

    self.assertEqual(
      self.portal.portal_simulation.getCurrentInventory(
        at_date=DateTime('3000/12/01 00:00:00 GMT+9'),
        resource_uid=self.resource.getUid(),
        section_uid=self.company.getUid(),
        node_uid=self.warehouse.getUid()),
      500)
    self.assertEqual(
      self.portal.portal_simulation.getFutureInventory(
        at_date=DateTime('3000/12/01 00:00:00 GMT+9'),
        resource_uid=self.resource.getUid(),
        section_uid=self.company.getUid(),
        node_uid=self.warehouse.getUid()),
      -2860)

  def afterSetUp(self):
    """
    This is ran before anything, used to set the environment
    """
    # categories
    self.cat = self.portal.portal_categories
    self.unit = self.cat.quantity_unit.unit
    self.time = self.cat.quantity_unit.time
    for id_ in ('piece', 'pallet'):
      if getattr(self.unit, id_, None) is None:
        self.unit.newContent(id=id_)

    # quantity_unit_conversion_module
    if getattr(self.portal.quantity_unit_conversion_module, 'unit_piece', None) is None:
      conversion_group = self.portal.quantity_unit_conversion_module.newContent(
        id='unit_piece',
        quantity_unit_value=self.unit.piece)
      conversion_group.validate()
      conversion_definition = conversion_group.newContent(
        quantity_unit_value=self.unit.pallet,
        quantity=1.0)
      conversion_definition.validate()

    # resource
    if getattr(self.portal.product_module, 'automated_ordering_test', None) is None:
      resource = self.portal.product_module.newContent(
        id='automated_ordering_test',
        title='AOTX',
        quantity_unit_value_list=[self.unit.piece,
                                  self.unit.pallet])
      resource.edit(individual_variation_base_category_list=[])
      conversion_group = resource.newContent(
        portal_type='Quantity Unit Conversion Group',
        quantity_unit_value=self.unit.piece)
      conversion_group.validate()
      conversion_definition = conversion_group.newContent(
        portal_type='Quantity Unit Conversion Definition',
        quantity_unit_value=self.unit.pallet,
        quantity=10.0)
      conversion_definition.validate()
    self.resource = self.portal.product_module.automated_ordering_test

    # organisations
    for id_ in ('automated_ordering_test_company',
                'automated_ordering_test_warehouse',
                'automated_ordering_test_supplier'):
      if getattr(self.portal.organisation_module, id_, None) is None:
        self.portal.organisation_module.newContent(id=id_)
    self.company = self.portal.organisation_module.automated_ordering_test_company
    self.warehouse = self.portal.organisation_module.automated_ordering_test_warehouse
    self.supplier = self.portal.organisation_module.automated_ordering_test_supplier

    # currency
    if getattr(self.portal.currency_module, 'automated_ordering_test', None) is None:
      currency = self.portal.currency_module.newContent(
        id='automated_ordering_test',
        title='Test',
        reference='Test',
      )
      currency.validate()
    self.currency = self.portal.currency_module.automated_ordering_test

    # purchase trade condition
    if getattr(self.portal.purchase_trade_condition_module, 'automated_ordering_test', None) is None:
      condition = self.portal.purchase_trade_condition_module.newContent(
        id='automated_ordering_test',
        specialise_value=self.portal.purchase_trade_condition_module['1'],# provided by configurator
        price_currency_value=self.currency,
        ledger_value=self.cat.ledger.automated_ordering,
        destination_section_value=self.company,
        source_section_value=self.supplier)
      condition.validate()
    self.condition = self.portal.purchase_trade_condition_module.automated_ordering_test

    # cancel existing orders
    for brain in self.portal.portal_catalog(
        portal_type='Purchase Order',
        default_ledger_uid=self.cat.ledger.automated_ordering.getUid(),
        **{'delivery.start_date':'>=3000'}):
      if not brain.getSimulationState() in ('draft', 'cancelled'):
        brain.cancel()

    self.commitTic()

    # initial stock
    self.setUpInitialStock()

    self.commitTic()

  def createPurchaseSupply(self):
    # delete existing supplies
    if getattr(self.portal.purchase_supply_module, 'automated_ordering_test', None) is not None:
      self.portal.purchase_supply_module.manage_delObjects(ids=['automated_ordering_test'])

    self.commitTic()

    supply =self.portal.purchase_supply_module.newContent(
      id='automated_ordering_test',
      price_currency_value=self.currency,
      source_section_value=self.supplier,
      source_value=self.supplier,
      destination_section_value=self.company,
      destination_value=self.warehouse,
      ledger_value=self.cat.ledger.automated_ordering,
      start_date_range_min=DateTime('3000/06/01 00:00:00 GMT+9'),
      start_date_range_max=DateTime('3001/01/01 00:00:00 GMT+9'))
    supply.validate()
    self.commitTic()
    return supply

  def runTest(self,
              expected_purchase_order_data_list,
              now,
              order_quantity_unit_value,
              time_quantity_unit_value,
              min_stock,
              max_delay,
              max_flow,
              min_order_quantity,
              max_order_quantity):
    self.createPurchaseSupply().newContent(
      portal_type='Purchase Supply Line',
      resource_value=self.resource,
      quantity_unit_value=self.unit.piece,
      order_quantity_unit_value=order_quantity_unit_value,
      time_quantity_unit_value=time_quantity_unit_value,
      min_stock=min_stock,
      max_flow=max_flow,
      min_order_quantity=min_order_quantity,
      max_order_quantity=max_order_quantity,
      max_delay=max_delay,
    )
    alarm = self.portal.portal_alarms.automated_ordering_alarm
    getattr(alarm.activate(), alarm.getActiveSenseMethodId())(now)
    self.commitTic()
    purchase_order_data_list = []
    for brain in self.portal.portal_catalog(
        portal_type='Purchase Order',
        default_ledger_uid=self.cat.ledger.automated_ordering.getUid(),
        simulation_state='planned',
        **{'delivery.start_date':'>=3000'}):
      order = brain.getObject()
      self.assertEqual(order.getLedgerValue(),
                       self.cat.ledger.automated_ordering)
      self.assertEqual(order.getSimulationState(), 'planned')
      self.assertEqual(order.getDestinationSectionValue(),
                       self.company)
      self.assertEqual(order.getDestinationValue(),
                       self.warehouse)
      self.assertEqual(order.getSourceSectionValue(),
                       self.supplier)
      self.assertEqual(order.getSourceValue(),
                       self.supplier)
      self.assertEqual(order.getSpecialiseValue(),
                       self.condition)
      self.assertEqual(order.getPriceCurrencyValue(),
                       self.currency)
      line_list = []
      for movement in order.getMovementList():
        line_list.append([
          movement.getStartDate().Date(),
          movement.getStopDate().Date(),
          movement.getResourceTitle(),
          movement.getQuantity(),
          movement.getQuantityUnitTitle(),
        ])
      line_list.sort()
      purchase_order_data_list.append(line_list)
    purchase_order_data_list.sort()
    self.assertEqual(purchase_order_data_list,
                     expected_purchase_order_data_list)

  def test_01(self):
    """Basic test
    """
    self.runTest(
      [[['3000/09/10', '3000/09/11', 'AOTX', 1460.0, 'Piece'],]],
      DateTime('3000/09/10 00:00:00 GMT+9'),
      self.unit.piece,
      self.time.day,
      min_stock=500,
      max_delay=1,
      max_flow=0,
      min_order_quantity=0,
      max_order_quantity=0,
    )

  def test_02(self):
    """Basic test
    """
    self.runTest(
      [[['3000/09/10', '3000/09/11', 'AOTX', 146.0, 'Pallet'],]],
      DateTime('3000/09/10 00:00:00 GMT+9'),
      self.unit.pallet,
      self.time.day,
      min_stock=500,
      max_delay=1,
      max_flow=0,
      min_order_quantity=0,
      max_order_quantity=0,
    )

  def test_03(self):
    """Basic test
    """
    self.runTest(
      [
        [['3000/09/10', '3000/09/11', 'AOTX',  46.0, 'Pallet'],
         ],
        [['3000/09/10', '3000/09/11', 'AOTX', 100.0, 'Pallet'],
         ],
      ],
      DateTime('3000/09/10 00:00:00 GMT+9'),
      self.unit.pallet,
      self.time.day,
      min_stock=500,
      max_delay=1,
      max_flow=0,
      min_order_quantity=0,
      max_order_quantity=100,
    )

  def test_04(self):
    """Basic test
    """
    self.runTest(
      [
        [['3000/09/10', '3000/09/11', 'AOTX',  50.0, 'Pallet'],
         ],
        [['3000/09/10', '3000/09/11', 'AOTX', 100.0, 'Pallet'],
         ],
      ],
      DateTime('3000/09/10 00:00:00 GMT+9'),
      self.unit.pallet,
      self.time.day,
      min_stock=500,
      max_delay=1,
      max_flow=0,
      min_order_quantity=50,
      max_order_quantity=100,
    )

  def test_05(self):
    """Basic test
    """
    self.runTest(
      [
        [['3000/09/10', '3000/09/11', 'AOTX',  50.0, 'Pallet'],
         ['3000/09/11', '3000/09/12', 'AOTX',  50.0, 'Pallet'],
         ['3000/09/12', '3000/09/13', 'AOTX',  46.0, 'Pallet'],
         ],
      ],
      DateTime('3000/09/10 00:00:00 GMT+9'),
      self.unit.pallet,
      self.time.day,
      min_stock=500,
      max_delay=1,
      max_flow=500,
      min_order_quantity=0,
      max_order_quantity=0,
    )

  def test_06(self):
    """Basic test
    """
    self.runTest(
      [
        [['3000/09/10', '3000/09/11', 'AOTX',  50.0, 'Pallet'],
         ['3000/09/11', '3000/09/12', 'AOTX',  50.0, 'Pallet'],
         ['3000/09/12', '3000/09/13', 'AOTX',  20.0, 'Pallet'],
         ],
        [['3000/09/12', '3000/09/13', 'AOTX',  26.0, 'Pallet'],
         ],
      ],
      DateTime('3000/09/10 00:00:00 GMT+9'),
      self.unit.pallet,
      self.time.day,
      min_stock=500,
      max_delay=1,
      max_flow=500,
      min_order_quantity=0,
      max_order_quantity=120,
    )

  def test_07(self):
    """Basic test
    """
    self.runTest(
      [
        [['3000/09/10', '3000/09/11', 'AOTX',  50.0, 'Pallet'],
         ['3000/09/11', '3000/09/12', 'AOTX',  50.0, 'Pallet'],
         ['3000/09/12', '3000/09/13', 'AOTX',  20.0, 'Pallet'],
         ],
        [['3000/09/12', '3000/09/13', 'AOTX',  30.0, 'Pallet'],
         ['3000/09/13', '3000/09/14', 'AOTX',  50.0, 'Pallet'],
         ],
      ],
      DateTime('3000/09/10 00:00:00 GMT+9'),
      self.unit.pallet,
      self.time.day,
      min_stock=500,
      max_delay=1,
      max_flow=500,
      min_order_quantity=80,
      max_order_quantity=120,
    )
